package com.mgh.pmdm.parques.view.ui

import android.os.Bundle
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.lifecycle.ViewModelProvider
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.mgh.pmdm.parques.view.dialogs.MiDialogFragment
import com.mgh.pmdm.parques.R
import com.mgh.pmdm.parques.databinding.FragmentFirstBinding
import com.mgh.pmdm.parques.db.Park
import com.mgh.pmdm.parques.viewmodel.MainViewModel

class FirstFragment : Fragment(), MiDialogFragment.OnOKOrCancelListener {

    // Propiedad privada para almacenar temporalmente un item
    // se utiliza cuando se va a eliminar un parque
    private var itemToRemove: Park?

    // Definimos una instancia del ViewModel como lateinit
    private lateinit var viewModel: MainViewModel

    // Definimos el data binding con la posibilidad de que
    // tenga valores nulos, y establecemos el getter
    private var _binding: FragmentFirstBinding? = null
    private val binding get() = _binding!!

    // Inicialización de propiedades
    init {
        itemToRemove = null
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {

        // Inyectamos la interfaz a través del Data Binding
        _binding = FragmentFirstBinding.inflate(inflater, container, false)

        return binding.root

    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // Instanciamos el ViewModel, utilizando la clase ViewModelProvider
        // En lugar de this, vamos a utilizar requireActivity(), para trabajar en el ámbito
        // de la actividad y poder compartir el ViewModel
        viewModel =
            ViewModelProvider(requireActivity()).get(MainViewModel::class.java)

        // Definimos el LayoutManager para el RecyclerView
        binding.ParksRecyclerView.layoutManager = LinearLayoutManager(requireActivity())

        // Suscribimos un observador al LiveData adaptador del ViewModel.
        // Cuando se produzcan cambios en el adaptadordel ViewModel,
        // se actualizará el adaptador del RecyclerView.

        // Nota: El Observer será el de la clase androidx.lifecycle

        viewModel.adaptador.observe(viewLifecycleOwner, {
            binding.ParksRecyclerView.adapter = it
        })

        // Observador del parkClicked del ViewModel.
        // Con ello sabemos el parque sobre el que se ha hecho click, para editarlo.
        viewModel.parkClicked.observe(viewLifecycleOwner, { park ->
            park?.let {
                // Reseteamos el valor de parkClicked
                viewModel.parkClicked.value = null

                // Actualizamos el parque actual del ViewModel
                viewModel.setCurrentPark(park)
                // Y lanzmos la navegación hacia el fragmento de edición
                findNavController().navigate(R.id.action_FirstFragment_to_SecondFragment)

            }
        })

        // Observador para el click largo sobre un item
        viewModel.parkLongClicked.observe(viewLifecycleOwner) { park ->
            park?.let {
                // Reseteamos el valor
                viewModel.parkLongClicked.value = null

                // Establecemos el elemento a eliminar
                itemToRemove = park

                // e Invocamos al diálogo para preguntar si deseamos eliminar el registro
                val miDialogo = MiDialogFragment(
                    resources.getString(R.string.deletePark),
                    resources.getString(R.string.askDeletePark) + park.name + "?",
                    this
                )

                miDialogo.show(requireActivity().supportFragmentManager, "miDialogo")

                // Más adelante se implementa el método onPositiveClick de la
                // interfaz MiDialogFragment.OnOKOrCancelListener, para gestionar
                // cuando se pulsa Aceptar en este diálogo de borrado

            }

        }

        // Observador para la lista de parques. Cuando esta toma valor, actualiza la vista.
        viewModel.parkList.observe(viewLifecycleOwner) { parks ->
            parks.let {
                if (viewModel.deletedPos.value == null) {
                    // Actualización general
                    viewModel.adaptador.value?.notifyDataSetChanged()
                } else {
                    // Actualización de borrado, para activar la animación
                    viewModel.adaptador.value?.notifyItemRemoved(viewModel.deletedPos.value!!)
                    viewModel.deletedPos.value=null
                }
            }
        }

    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    /* Otros métodos*/

    // Métodos para implementar la interfaz MiDialogFragment.OnOKOrCancelListener
    override fun onPositiveClick() {
        // Si se pulsa en Aceptar en el diálogoremovePark
        // eliminamos el item.
        itemToRemove?.also {
            viewModel.removePark(it)
        }
    }

    override fun onCancelClick() {
        Toast.makeText(
            requireActivity().applicationContext,
            resources.getString(R.string.actionCancelled), Toast.LENGTH_SHORT
        ).show()
    }

}